//==============================================================================
/* 
Este programa realiza una simulacion del tallado de engranajes rectos. 
Simula la generacion de engranajes rectos normalizados.
Simula la generacion de engranajes rectos indicando los parametros
constructivos de los mismos.
Simula la generacion de engranajes rectos segun la norma ISO 6336
Simula la generacion de engranajes rectos asimetricos
Simula la generacion de engranajes rectos con dientes de cremallera circulares

Copyright (C) 2011 Daniel Costas Rodriguez

Este programa es software libre: usted puede redistribuirlo y/o modificarlo 
bajo los trminos de la Licencia Pblica General GNU publicada por la Fundacin
para el Software Libre, ya sea la versin 3 de la Licencia, o (a su eleccin)
cualquier versin posterior.

Este programa se distribuye con la esperanza de que sea til, pero SIN GARANTIA
ALGUNA; ni siquiera la garanta implcita MERCANTIL o de APTITUD PARA UN
PROPSITO DETERMINADO. Consulte los detalles de la Licencia Pblica General
GNU para obtener una informacion ms detallada.

Debera haber recibido una copia de la Licencia Pblica General GNU junto a 
este programa.
 */
//==============================================================================



#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>				// OpenGL
#include <GL/glut.h>			// The OpenGL Utility Toolkit
#include <GL/glui.h>			// GLUI User Interface Library
#include "mecanismos.h"			// Definicion de los dientes de cremallera
#include <math.h>				// Libreria matematica
#include <time.h>				// Libreria para gestion de tiempos



//==============================================================================
//	Constantes
//==============================================================================

// Valor nulo
#define NADA_ID 				0

// OBJETO DEL SIMULADOR: Fondo
#define FONDO_ID				0

// OBJETO DEL SIMULADOR: Engranaje
#define ENGRANAJE_ID			1	

// OBJETO DEL SIMULADOR: Cremallera
#define CREMALLERA_ID			2

// OBJETO DEL SIMULADOR: Circulo de fondo
#define CIRCULO_FONDO_ID		3

// OBJETO DEL SIMULADOR: Circulo de paso
#define CIRCULO_PASO_ID			4


// Id correspondientes al movimiento de la cremallera

// MOVIMIENTO DE LA CREMALLERA: Sentido creciente del eje Y
#define SUBIR_ID				10

// MOVIMIENTO DE LA CREMALLERA: Sentido creciente del eje X
#define DERECHA_ID				11

// MOVIMIENTO DE LA CREMALLERA: Sentido decreciente del eje Y
#define BAJAR_ID				12

// MOVIMIENTO DE LA CREMALLERA: Sentido decreciente del eje X
#define IZQUIERDA_ID			13

// MOVIMIENTO DE LA CREMALLERA: La cremallera ha terminado el tallado
#define FIN_ID					14

// MOVIMIENTO DE LA CREMALLERA: La cremallera esta en la ultima posicion
#define OFF_ID					15

// Id correspondientes a glui_control
#define SIMULADOR_ID			1
#define SIMU_NORMA_ID			100
#define SIMU_TRAPECIO_ID		110

// Id correspondientes a v_glui_control
#define CERRAR_ID				1000
#define DATOS_DIENTES_ID		1010
#define DATOS_DIAMETRO_EXT_ID	1020
#define DATOS_DIAMETRO_INT_ID	1030
#define DATOS_DIAMETRO_PRI_ID	1040
#define DATOS_ANGULO_ID			1050
#define DATOS_ANGULO_2_ID		1060
#define DATOS_MODULO_ID			1070
#define DATOS_PENETRACION_ID	1080
#define DATOS_ISO_RADIO_ID		1090
#define DATOS_ISO_DEPRE_ID		1100
#define DATOS_FLECHA_ID			1110
#define DATOS_ELEMENTOS_ID		1120
#define DATOS_GIRO_ID			1130
#define MODO_SIMULAR_ID			1140
#define MOVER_ADELANTE_ID		1150
#define MOVER_ATRAS_ID			1160
#define CLEAR_ID				1170
#define SALIR_ID				1180

#define COMPONENTE_ROJO_ID		5000
#define COMPONENTE_VERDE_ID		5100
#define COMPONENTE_AZUL_ID		5200

#define MOVER_XY_ID				7000
#define MOVER_Z_ID				7010


//==============================================================================
//	Variables
//==============================================================================

// Efecto zoom en la imagen del simulador
float trans[3] = {0.0, 0.0, 0.0};
int mouse_x, mouse_y, mouse_z;
int MoverGrafico;

// Variable que contiene el tipo de simulacion que se esta realizando
int QueSimula = 0;

// Variable que contiene el objeto al que apunta el selector de color
int QueObjeto = FONDO_ID;


// DATOS DEL ENGRANAJE

// ENGRANAJE Radio de la circunferencia base
float Rb;

// ENGRANAJE Radio de la circunferencia primitiva
float Rp;

// ENGRANAJE Radio de la circunferencia exterior
float Rc;

// ENGRANAJE Numero de dientes
float Z;

// ENGRANAJE Paso circular
float Paso;

// ENGRANAJE Angulo de presion
float Angulo, Angulo_2;

// ENGRANAJE Contador de los dientes realizados en la cremallera
int Diente;


// DATOS DE LA CREMALLERA

// CREMALLERA Origen del sistema de referencia
float Origen;

// CREMALLERA Indicador del lugar donde esta trabajando la cremallera
int C_Mueve;

// CREMALLERA Guarda la ultima posicion X antes del cambio de sentido
float C_Ultimo_X;

// CREMALLERA Indicador del desplazamiento vertical de la cremallera
float C_Desplaza;

// CREMALLERA Posicion segun el eje X
float C_X;

// CREMALLERA Posicion segun el eje Y
float C_Y;

// CREMALLERA Incremento vertical
float C_Paso_Y;

/* CREMALLERA Vector en el que se guarda la posicion de la cremallera en 
   funcion del angulo girado por el engranaje */
float *C_Historia_X;

/* CREMALLERA Contador de las posiciones ocupadas por la cremallera en el 
   tallado del engranaje */
int Conta_h = 1;

/* CREMALLERA Vector que indica de la cremallera los siguientes datos:
        - [0] posicion respecto al eje X
        - [1] la posicion respecto al eje Y
        - [2] el angulo girado respecto al eje Z
        - [3] la altura del diente
        - [4] el paso de la cremallera
        - [5] el angulo de presion
        - [6] desplazamiento de la cremallera para la penetracion
        - [7] ISO 6336 radio de cabeza del diente
        - [8] ISO 6336 depresion lateral del diente
		- [9] Flecha de la deformacion circular
		- [10] valor del segundo angulo de presion para dientes asimetricos
 */
float C_Datos[11];

// CREMALLERA ISO 6336 Radio de cabeza
float ISO_Radio;

// CREMALLERA ISO 6336 Depresion lateral
float ISO_Depresion;

// CREMALLERA Valor de la flecha de deformacion del diente
float Flecha = 0;


// DATOS DEL PROCESO

/* PROCESO Indicador de la forma
         - true = automatico
         - false = manual
 */
int Stop = true;

// PROCESO En proceso manual si vale 1 es un movimiento hacia adelante
// 		 si vale -1 movimiento hacia atras
int Movimiento = 0;

// PROCESO Indica que es el primer corte que se da al engranaje
int Primera_Vez = true;

// PROCESO Contador de los movimientos realizados por la cremallera
int C_Pasos;

//int C_Contador_X;
float C_Contador;

// PROCESO Contador del angulo girado (tallado) del engranaje
float E_angulo;

// PROCESO Angulo que gira el engranaje al ser tallado
float E_incremento;

// PROCESO Indicador de visibilidad del efecto ISO
int C_ISO;

// PROCESO Indicador de visibilidad del efecto experimental
int C_Flecha;

// PROCESO Indicador de visibilidad del efecto de diente asimetrico
int C_Asimetria;

// PROCESO Indicador de visibilidad de la circunferencia de paso
int C_Paso_ON;

// PROCESO Indicador de visibilidad de la circunferencia de fondo
int C_Fondo_ON;

/* PROCESO Indicador del desplazamiento de la talla del engranaje para generar
   dientes gruesos (reforzados) por la base o aligerados */
float Penetracion = 0.0;


// COLORES
float ROJO[] = {1.0, 0.0, 0.0};
float BLANCO[] = {1.0, 1.0, 1.0};
float AMARILLO[] = {1.0, 1.0, 0.0};
float NEGRO[] = {0.0, 0.0, 0.0};
float AZUL[] = {0.0, 0.0, 1.0};

float ColorFondo[] = {0.0, 0.0, 0.0}; 				// negro
float ColorEngranaje[] = {1.0, 0.0, 0.0}; 			// rojo
float ColorCremallera[] = {0.0, 0.0, 1.0}; 			// azul
float ColorCirFondo[] = {1.0, 1.0, 0.0}; 			// amarillo
float ColorCirPaso[] = {1.0, 1.0, 1.0}; 			// blanco

// Modulos recomendados por la norma ISO 54:1997
const char *MODULO[] = {"1.00", "1.25", "1.50", "2.00", "2.50", "3.00", "4.00",
    "5.00", "6.00", "8.00", "10.00", "12.00", "16.00", "20.00", "25.00",
    "32.00", "40.00", "50.00"};

// Variables para el temporizador
long incio, fin;

int winIdMain;

// Variables correspondientes a objetos GLUI
GLUI *glui = NULL;
GLUI *v_glui = NULL;

GLUI_Panel *glui_firma, *glui_datos, *glui_iso, *glui_experimental, 
		   *glui_circular, *glui_asimetrico, *glui_opciones, *glui_proceso, 
		   *glui_boton, *glui_zoom;

GLUI_EditText *glui_RCabeza, *glui_RBase, *glui_Dientes, *glui_Angulo, 
			  *glui_Angulo_2, *glui_Penetracion, *glui_Flecha, *glui_radio, 
			  *glui_depresion, *glui_Giro, *glui_RPaso, *glui_Modulo_T, 
			  *glui_EjeY, *glui_EjeX;

GLUI_RadioGroup *glui_Proceso, *glui_Elementos;

GLUI_RadioButton *glui_Continuo, *glui_PasoAPaso;

GLUI_RadioButton *glui_Fondo, *glui_Engranaje, *glui_Cremallera, *glui_C_Fondo, 
				 *glui_C_Paso;

GLUI_Spinner *glui_Rojo, *glui_Verde, *glui_Azul;

GLUI_Button *glui_AutoOnOff, *glui_Sig, *glui_Ant, *glui_Clear;

GLUI_StaticText *informa, *aviso;

GLUI_Checkbox *glui_ISO, *glui_C_Flecha, *glui_C_Asimetria, *glui_CPaso, 
			  *glui_CFondo;

GLUI_Listbox *glui_Modulo;

GLUI_Rollout *glui_Color;



//==============================================================================
//	Funciones
//==============================================================================


//------------------------------------------------------------------------------
//  Nombre:	initDisplay
//  Funcion:	Inicializa las caracteristicas de la ventana de simulacion
//  		e incluye los procedimientos necesarios para iniciar su uso
//------------------------------------------------------------------------------

void initDisplay(void) {

	// 	- Selecciona el color negro para limpiar el buffer
    glClearColor(ColorFondo[0], ColorFondo[1], ColorFondo[2], 1);

    //	- Limpiar la ventana
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /*	- La vista es ortogonal centrada en (0,0) y
          lado (4 * Radio de cabeza) del engranaje
     */
    glOrtho(-2 * Rc, 2 * Rc, -2 * Rc, 2 * Rc, 1, -1);

    if (glutGetWindow() != winIdMain) {
        glutSetWindow(winIdMain);
    }

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		mainDisplay
//	Funcion:	Actualizar la ventana grafica (RenderScene)
//------------------------------------------------------------------------------

void mainDisplay(void) {

    // leemos el tiempo del sistema
    long inicio = clock();

    //		- Limpiar la ventana
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    //		- Reinicia el sistema de coordenadas
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // 		- La vista de la ventana es ortogonal
    glOrtho(1.4 * Origen - trans[0] - trans[2],
            -1.4 * Origen - trans[0] + trans[2],
            1.4 * Origen - trans[1] - trans[2],
            -1.4 * Origen - trans[1] + trans[2],
            1, -1);

    //		- Reinicia el sistema de modelado
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    //		- Dibuja el engranaje
	if ((Penetracion < 0) && (QueSimula == SIMU_NORMA_ID)) {
    	Cilindro(Rc - Penetracion, ColorEngranaje);
	} else {
	    Cilindro(Rc, ColorEngranaje);
	}

    //		- Dibuja la circunferencia primitiva
    if (C_Paso_ON) {
        Circunferencia(Rp, ColorCirPaso);
    }

    //		- Dibuja la circunferencia de fondo
    if (C_Fondo_ON) {
        if (QueSimula == SIMU_NORMA_ID) {
	       Circunferencia(Rb - Penetracion, ColorCirFondo);
        } else {
          Circunferencia(Rb, ColorCirFondo);
        }
    }

    // Parametros del diente de cremallera
    C_Datos[0] = 0.0; 				// X del centro de la cremallera
    C_Datos[1] = 0.0; 				// Y del centro de la cremallera
    C_Datos[2] = 0.0; 				// angulo girado en el sentido de eje z
    C_Datos[3] = Rc - Rb; 			// Altura del diente
    C_Datos[4] = Paso; 				// Paso circular
    C_Datos[5] = Angulo; 			// Angulo de presion

	// Se hacen visibles o no las modificaciones ISO
	if (C_ISO) {
	    C_Datos[7] = ISO_Radio; 		// ISO 6336 radio de cabeza
	    C_Datos[8] = ISO_Depresion; 	// ISO 6336 depresion lateral
	} else {
		C_Datos[7] = 0;
		C_Datos[8] = 0;
	}

	// Se hace visible o no la asimetria en los dientes de cremallera
	if (C_Asimetria) {
		C_Datos[10] = Angulo_2;
	} else {
		C_Datos[10] = Angulo;
	}

    // Si existe penetracion hay que separar el diente del eje de rotacion
    // un valor igual a la penetracion
    float avance;
    if (Penetracion > 0) {
        C_Datos[6] = Penetracion;
        avance = Rc;
    } else {
        C_Datos[6] = 0;
        avance = Rc - Penetracion;
    }

	// Si existe deformacion circular del lateral del diente
	if (C_Flecha) {
		C_Datos[9] = Flecha; 
	} else {
		C_Datos[9] = 0;
	}

    //		- Dibuja las cremalleras que han mordido el engranaje
    float mma = E_angulo;
    int cuenta_angulo = 0;
    int mmax = 0;
    while (mma > 0) {
        C_Datos[0] = avance * sin(mma * M_PI / 180) +
                C_Historia_X[cuenta_angulo] * cos(mma * M_PI / 180);
        C_Datos[1] = Origen + C_Desplaza +
                avance * (1 - cos(mma * M_PI / 180)) +
                C_Historia_X[cuenta_angulo] * sin(mma * M_PI / 180);
        C_Datos[2] = mma;
		Cremallera(ColorFondo, C_Datos);
        mma -= E_incremento;
        cuenta_angulo++;
    }

    //		- Dibujar la cremallera visible
    C_Datos[0] = C_X;
    C_Datos[1] = C_Y + Origen;
    C_Datos[2] = 0;
	Cremallera(ColorCremallera, C_Datos);

	// Se hace visible en pantalla el renderizado
    if (glutGetWindow() != winIdMain) {
        glutSetWindow(winIdMain);
    }

	glutPostRedisplay();
    glutSwapBuffers();

    // Temporizador
    do {
        fin = clock();
    } while (difftime(fin, inicio) < 60);
    
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		idle
//	Funcion:	There can be only one idle() callback function. In an 
//			animation this idle() function must update not only
//			the main window but also all derived subwindows
//------------------------------------------------------------------------------

void idle(void) {

    // Incremento segun el eje X por cada giro del engranaje
    static float C_Xx;

    // Desplazamiento horizontal de la cremallera cuando no esta tallando
    static int C_Contador_X;

    if (!Stop || (Movimiento == 1)) {
        
		// paso hacia adelante
        switch (C_Mueve) {

            case SUBIR_ID:
       
				if (C_Contador < 10) {
               
					if (Primera_Vez) {
                   
						C_Desplaza += C_Paso_Y;

                        if (C_Contador == 0) {
                       
                        	// Se habilita el boton para ir hacia atras solo si
							// esta habilitado el modo manual
							if (Stop) {
                                glui_Ant->enable();
                            }
                           
							// Se deshabilita la entrada de valores opcionales
                            // Numero de dientes
                            glui_Dientes->disable();
                           
							// Diametro circunferencia primitiva
                            glui_RPaso->disable();
                           
							// Diametro exterior
                            glui_RCabeza->disable();
                           
							// Diametro interior
                            glui_RBase->disable();
                           
							// Angulo de presion
                            glui_Angulo->disable();
                           
							// Angulo girado por tallada
                            glui_Giro->disable();
                           
							// Penetracion
                            glui_Penetracion->disable();
                           
							// Angulo girado en cada tallada
                            glui_Giro->disable();
                           
							switch (QueSimula) {
                           
								case SIMU_NORMA_ID:
                               
									// Modulo en listbox
                                    glui_Modulo->disable();
                                   
									break;

                                case SIMU_TRAPECIO_ID:
                                   
									// Modulo en textbox
                                    glui_Modulo_T->disable();
                                   
									// Flecha de deformacion del diente
                                    glui_Flecha->disable();
                                   
									break;

                            }
                        }
                    }

                    C_Y += C_Paso_Y;
                    C_Contador++;
                    Movimiento = 0;

                } else {

                    C_Mueve = DERECHA_ID;
                    C_Xx = Rp * sin(E_incremento * M_PI / 180);
                    C_Ultimo_X = C_X;
                    Primera_Vez = false;

                }

                break;


            case DERECHA_ID:

                if (C_X < Paso) {

                    E_angulo += E_incremento;
                    C_X += C_Xx;
                    C_Historia_X[Conta_h] = C_X;
                    Conta_h++;
                    Movimiento = 0;
                    if (E_angulo > 360) {
                        C_Mueve = BAJAR_ID;
                    }

                } else {

                    Diente += 2;
                    if (Diente < Z + 1) {
                        C_Xx = Paso / 10;
                        C_Contador_X = 20;
                    }
                    C_Mueve = BAJAR_ID;

                }

                break;

            case BAJAR_ID:

                if (C_Contador > 0) {

                    C_Y -= C_Paso_Y;
                    C_Contador--;
                    Movimiento = 0;

                } else {

                    if (E_angulo >= 360) {
                        glui_Sig->disable();
                        C_Mueve = FIN_ID;
                    } else {
                        C_Mueve = IZQUIERDA_ID;
                    }

                }

                break;

            case IZQUIERDA_ID:
				
                if (C_Contador_X > 0) {

                    C_X -= C_Xx;
                    Movimiento = 0;
                    C_Contador_X--;

                } else {

                    C_Mueve = SUBIR_ID;

                }

                break;

            case FIN_ID:

                Stop = true;
                glui_AutoOnOff->set_name(const_cast<char*> ("Automatico"));
                glui_AutoOnOff->disable();
                glui_Ant->enable();
                C_Mueve = OFF_ID;
                Movimiento = 0;

                break;

            default:
                
				break;

        }

    }

    if (Movimiento == -1) {

        switch (C_Mueve) {

            case SUBIR_ID:

                if (C_Contador > 0) {

                    if (Primera_Vez) {
                        C_Desplaza -= C_Paso_Y;
                    }
                    C_Y -= C_Paso_Y;
                    C_Contador--;
                    Movimiento = 0;

                    if ((C_Contador == 0) && Primera_Vez) {
                        // Llegados al origen del movimiento no es posible
                        // retroceder mas.
                        glui_Ant->disable();
                    }

                } else {

                    C_Mueve = IZQUIERDA_ID;
                    C_Xx = Paso / 10;

                    C_Ultimo_X = C_X;
                    C_Contador_X = 0;

                }

                break;

            case IZQUIERDA_ID:

                if ((C_Contador_X < 20) || (Primera_Vez && C_Contador_X < 10)) {
                    
					C_X += C_Xx;
                    C_Contador_X++;
                    Movimiento = 0;

                } else {

                    C_Mueve = BAJAR_ID;

                }

                break;

            case BAJAR_ID:

                if (C_Contador < 10) {

                    C_Y += C_Paso_Y;
                    C_Contador++;
                    Movimiento = 0;

                } else {

                    C_Mueve = DERECHA_ID;
                    C_Xx = Rp * sin(E_incremento * M_PI / 180);

                }

                break;

            case DERECHA_ID:

                if ((C_X > C_Ultimo_X) && (E_angulo > 0)) {

                    E_angulo -= E_incremento;
                    C_X -= C_Xx;
                    C_Historia_X[Conta_h] = 0;
                    Conta_h--;

                    // Si llegamos a la primer posicion del tallado del
                    // engranaje se activa el indicador de que es la primera vez
                    if (Conta_h == 1) {
                        Primera_Vez = true;
                    }

                    Movimiento = 0;

                } else {

                    C_Mueve = SUBIR_ID;
					
                }

                break;

            case OFF_ID:

                // Estamos al final del proceso y se quiere dar un paso hacia
                // atras, con lo que se habilitan los botones y se desplaza
                // la cremallera una posicion
                glui_AutoOnOff->enable();
                glui_Sig->enable();
                C_Mueve = BAJAR_ID;
                C_Y += C_Paso_Y;
                C_Contador++;
                Movimiento = 0;

                break;

            default:

                break;

        }

    }

    // actualiza main y sub ventanas
    if (glutGetWindow() != winIdMain) {
        glutSetWindow(winIdMain);
    }

    glutPostRedisplay();

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		mainReshape
//	Funcion:	Callback function de redibujado de la ventana si se 
//			cambia 	de tamao.
//
//	Entradas:
//		w	ancho de la nueva ventana
//		h	alto de la nueva ventana
//------------------------------------------------------------------------------

void mainReshape(int w, int h) {

    initDisplay();

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		keyboard
//	Funcion:	Manejar las entradas por el teclado
//------------------------------------------------------------------------------

void keyboard(unsigned char key, int x, int y) {

    /*
     * De meter codigos de teclado, hay que controlar que la operacion a 
     * ejecutar sea posible en el momento determinado. No se puede pulsar una
     * tecla que haga ir hacia atras el simulador cuando esta en la primera
     * posicion de la simulacion ya que no existen pasos hacia atras.
     */

    if (glutGetWindow() != winIdMain) {
        glutSetWindow(winIdMain);
    }

    glutPostRedisplay();

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		mousePress y activeMouseMotion
//	Funcion:	Manejar las entradas por del raton
//------------------------------------------------------------------------------

void mousePress(int button, int state, int x, int y) {

    int specialKey = glutGetModifiers();

	if (state == GLUT_UP) {
        MoverGrafico = NADA_ID;
    }

    if (state == GLUT_DOWN) {

        if (button == GLUT_LEFT_BUTTON) {

            if (specialKey == GLUT_ACTIVE_CTRL) {

                MoverGrafico = MOVER_Z_ID;
                mouse_y = y;

            } else {

                MoverGrafico = MOVER_XY_ID;
                mouse_x = x;
                mouse_y = y;

            }

        }

        if (button == GLUT_RIGHT_BUTTON) {

            MoverGrafico = MOVER_Z_ID;
            mouse_y = y;

        }

    }

}


void activeMouseMotion(int x, int y) {

    if (MoverGrafico == MOVER_XY_ID) {

        trans[0] += (float) (x - mouse_x) / 18;
        trans[1] += (float) (mouse_y - y) / 18;
        mouse_x = x;
        mouse_y = y;
        glutPostRedisplay();

    }

    if (MoverGrafico == MOVER_Z_ID) {

        trans[2] += (float) (mouse_y - y) / 18;
        mouse_y = y;
        glutPostRedisplay();

    }

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		pressKey y releaseKey
//	Funcion:	Manejar las entradas por teclado
//------------------------------------------------------------------------------

void pressKey(int key, int x, int y) {

}


void releaseKey(int key, int x, int y) {

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		v_glui_control
//	Funcion:	Manejar los eventos generados por la ventana de
//			simulacion
//
//	Entrada:
//		control		valor correspondiente a cada control
//------------------------------------------------------------------------------

void v_glui_control(int control) {

	float tpAngulo, tpLimite;

	switch (control) {

        case CLEAR_ID:

            // Se reinician las variables
            Primera_Vez = true;
            C_Y = 0;
            C_Contador = 0;
            C_Mueve = SUBIR_ID;
            C_X = 0;
            Conta_h = 1;
            E_angulo = 0;
            C_Desplaza = 0;
            Diente = 0;
            Stop = true;
			trans[0] = 0.0;
			trans[1] = 0.0;
			trans[2] = 0.0;
           

			// Se reinician los botones del tablero
            glui_Sig->enable();
            glui_AutoOnOff->enable();
            glui_AutoOnOff->set_name(const_cast<char*> ("Automatico"));
          

			// Se habilitan los campos de entrada de datos
            glui_Dientes->enable(); 			// Numero de dientes
            glui_Dientes->set_int_val(14);
            Z = 14;
   
			glui_RPaso->enable(); 				// Circunferencia primitiva
            glui_RPaso->set_float_val(14);
            Rp = 7;
            
			glui_RCabeza->enable(); 			// Circunferencia exterior
            glui_RCabeza->set_float_val(16);
            Rc = 8;
           
			glui_RBase->enable(); 				// Circunferencia interior
            glui_RBase->set_float_val(11.5);
            Rb = 5.75;
           
			glui_Angulo->enable(); 				// Angulo de presion
            glui_Angulo->set_float_val(20);
            Angulo = 20;
            
			glui_Penetracion->enable(); 		// Penetracion
            glui_Penetracion->set_float_val(0);
			Penetracion = 0;
            
			glui_Giro->enable(); 				// Giro en cada tallada
            glui_Giro->set_int_val(1);
           
			E_incremento = 1;

            switch (QueSimula) {

                case SIMU_NORMA_ID:

                    glui_Modulo->enable(); 		// Modulo (listbox)
                    glui_Modulo->set_int_val(0);
            		Paso = 2 * M_PI * Rp / Z;
                   
					break;

                case SIMU_TRAPECIO_ID:

                    glui_Modulo_T->enable(); 	// Modulo (texto)
                    glui_Modulo_T->set_float_val(1.0);
                    Paso = M_PI * glui_Modulo_T->get_float_val();
          

					// Se reinicia el panel ISO
                    glui_iso->enable();
                    glui_radio->set_float_val(0);
					ISO_Radio = 0;
           
					glui_depresion->set_float_val(0);
					ISO_Depresion = 0;
					glui_depresion->disable();
                  

					// Se reinicia el panel Experimental    
                    glui_experimental->enable();
                    glui_Flecha->set_float_val(0);
					Flecha = 0;
                    
					break;
            }

            C_Paso_Y = 2 * (Rc - Rb) / 10;


			// Se comprueba si hay que poner aviso de socavacion del diente
			if (QueSimula == SIMU_NORMA_ID) {
				if ( (Z < (2 / pow(sin(Angulo * M_PI / 180), 2))) && 
					(-Penetracion < 
					 (1 - Z / 2 * pow(sin(Angulo * M_PI / 180), 2))) ) {
					informa->set_text("    EXISTE SOCAVACION DEL DIENTE");
				} else {
					informa->set_text("");
				}
			}

            break;


        case DATOS_DIENTES_ID:

            // El numero de dientes ha cambiado. Se reajusta el resto de los
            // parametros para que queden normalizados
            Z = glui_Dientes->get_int_val();

			// El numero de dientes minimo es 3
			if (Z < 3) {
				glui_Dientes->set_int_val(3);
				Z = 3;
			}

			if (QueSimula == SIMU_NORMA_ID) {

                    // Se recalculan los parametros para que quede normalizado
                    Rp = atof(MODULO[glui_Modulo->get_int_val()]) * Z / 2;
                    Rc = Rp * (1 + 2 / Z);
                    Rb = Rp * (1 - 2.5 / Z);
                
					// Se indican los nuevos valores en la ventana
                    glui_RCabeza->set_float_val(Rc * 2);
                    glui_RBase->set_float_val(Rb * 2);
                    glui_RPaso->set_float_val(Rp * 2);

            }
           
			// Se recalcula el origen de la cremallera, el paso
            // vertical y el paso
            Origen = -Rc - 2 * (Rc - Rb);

            if (Penetracion > 0) {
                // La penetracion positiva se realiza desplazando
                // el diente desde el eje de rotacion
                C_Paso_Y = 2 * (Rc - Rb) / 10;
            } else {
                // La penetracion negativa se realiza disminuyendo
                // el desplazamiento vertical del eje de rotacion
                C_Paso_Y = (2 * (Rc - Rb) + Penetracion) / 10;
            }

            Paso = 2 * M_PI * Rp / Z;
            
			// Se comprueba si hay que poner aviso de socavacion del diente
			if (QueSimula == SIMU_NORMA_ID) {
				if ( (Z < (2 / pow(sin(Angulo * M_PI / 180), 2))) && 
					(-Penetracion < 
					 (1 - Z / 2 * pow(sin(Angulo * M_PI / 180), 2))) ) {
					informa->set_text("    EXISTE SOCAVACION DEL DIENTE");
				} else {
					informa->set_text("");
				}
			}

            break;


        case DATOS_DIAMETRO_PRI_ID:

            // El diametro de la circunferencia primitiva ha cambiado. Se
            // reajusta el resto de los parametros para que quede normalizado.
            Rp = glui_RPaso->get_float_val() / 2;

			// El diametro no puede tomar valores negativos
			if (Rp < 0) {

				glui_RPaso->set_float_val(0);
				Rp = 0;

			}

			if (QueSimula == SIMU_NORMA_ID) {

				Rc = Rp * (1 + 2 / Z);
                Rb = Rp * (1 - 2.5 / Z);
           
				// Se indican los nuevos valores en la ventana
                glui_Modulo->set_float_val(2 * Rp / Z);
                glui_RCabeza->set_float_val(Rc * 2);
                glui_RBase->set_float_val(Rb * 2);
            	Paso = 2 * M_PI * Rp / Z;

            }

            // Se recalcula el origen de la cremallera, el paso
            // vertical y el paso.
            Origen = -Rc - 2 * (Rc - Rb);
            C_Paso_Y = 2 * (Rc - Rb) / 10;

            break;


        case DATOS_MODULO_ID:

            // Se ha modificado el valor del modulo. Se procede a actualizar los
            // diametros primitivo, exterior e interior
            switch (QueSimula) {

                case SIMU_NORMA_ID:
                
					Rp = Z / 2 * atof(MODULO[glui_Modulo->get_int_val()]);
                    Rc = Rp * (1 + 2 / Z);
                    Rb = Rp * (1 - 2.5 / Z);
            		Paso = 2 * M_PI * Rp / Z;
           
					// Se indican los nuevos valores en la ventana
                    glui_RPaso->set_float_val(Rp * 2);
                    glui_RCabeza->set_float_val(Rc * 2);
                    glui_RBase->set_float_val(Rb * 2);
                   
					break;


                case SIMU_TRAPECIO_ID:

					// El modulo tiene que estar por encima de 1
					if (glui_Modulo_T->get_float_val() < 1) {
						glui_Modulo_T->set_float_val(1);
					}
                   
					Paso = M_PI * glui_Modulo_T->get_float_val();

                    break;


                default:

                    break;

            }

            // Se recalcula el origen de la cremallera, el paso vertical.
            Origen = -Rc - 2 * (Rc - Rb);
            C_Paso_Y = 2 * (Rc - Rb) / 10;

            break;


        case DATOS_DIAMETRO_EXT_ID:

            // Se ha modificado el valor del diametro exterior. Se actualiza la
            // variable Rc
            Rc = glui_RCabeza->get_float_val() / 2;

			// El valor del diametro tiene que ser positivo
			if (Rc < 0) {

				glui_RCabeza->set_float_val(0);
				Rc = 0;

			}

			if (QueSimula == SIMU_NORMA_ID) {
				
				// El diametro exterior no puede ser menor que el diametro
                // de paso
                if (Rc < Rp) {
                	Rc = Rp;
                    glui_RCabeza->set_float_val(Rc * 2);
                }

				Paso = 2 * M_PI * Rp / Z;
            }

            // Se recalcula el origen de la cremallera, el paso
            // vertical y el paso.
            Origen = -Rc - 2 * (Rc - Rb);
            C_Paso_Y = 2 * (Rc - Rb) / 10;
            
			break;


        case DATOS_DIAMETRO_INT_ID:

            // Se ha modificado el valor del diametro interior. Se actualiza la
            // variable Rb
            Rb = glui_RBase->get_float_val() / 2;

			// Su valor no puede ser negativo
			if (Rb < 0) {

				glui_RBase->set_float_val(0);
				Rb = 0;

			}
            
			// El diametro interior no puede ser mayor que el diametro de paso
            if (Rb > Rp) {

                Rb = Rp;
                glui_RBase->set_float_val(Rb * 2);

            }
            
			// Se recalcula el origen de la cremallera, el paso vertical y
            // el paso.
            Origen = -Rc - 2 * (Rc - Rb);
            if (Penetracion > 0) {
               C_Paso_Y = 2 * (Rc - Rb) / 10;
            } else {
               C_Paso_Y = (2 * (Rc - Rb) + Penetracion) / 10;
            }       
            if (QueSimula == SIMU_NORMA_ID) { 
				Paso = 2 * M_PI * Rp / Z;
			}

			break;


        case DATOS_ANGULO_ID:

            // Se ha modificado el angulo
            Angulo = glui_Angulo->get_float_val();
        
			// Se comprueba si hay que poner aviso de socavacion del diente
			if (QueSimula == SIMU_NORMA_ID) {
				if ( (Z < (2 / pow(sin(Angulo * M_PI / 180), 2))) && 
					(-Penetracion < 
					 (1 - Z / 2 * pow(sin(Angulo * M_PI / 180), 2))) ) {
					informa->set_text("    EXISTE SOCAVACION DEL DIENTE");
				} else {
					informa->set_text("");
				}
			}

			break;


        case DATOS_PENETRACION_ID:

             // Se ha modificado el coeficiente de desplazamiento ,por lo que 
            // se procede a ajustar el valor de la variable Penetracion que
            // mide el desplazamiento de la cremallera.
            //Penetracion = glui_Penetracion->get_float_val();
            Penetracion = -(glui_Penetracion->get_float_val() * Paso / M_PI);
            
			// Calculo del paso vertical
            if (Penetracion > 0) {

                // La penetracion positiva se realiza desplazando
                // el diente desde el eje de rotacion
                C_Paso_Y = 2 * (Rc - Rb) / 10;
      
                // El radio base aparente se desplaza hacia el centro
                // del cilindro, aunque a efectos de calculo no varie
                glui_RBase->set_float_val(Rb * 2 - Penetracion);          
                
                if (QueSimula == SIMU_TRAPECIO_ID) {
                   Rb = Rb - Penetracion;
                   Penetracion = 0;
                }           
			

            } else {

               // La penetracion negativa se realiza disminuyendo
                // el desplazamiento vertical del eje de rotacion
                C_Paso_Y = (2 * (Rc - Rb) + Penetracion) / 10;      
          
               // El radio de cabeza aparente se desplaza hacia afuera
               // del engranaje, aunque a efectos de calculo no varie
               glui_RCabeza->set_float_val(Rc * 2 - Penetracion);
                
                if (QueSimula == SIMU_TRAPECIO_ID) {
                   Rc = Rc - Penetracion;
                   Penetracion = 0;
                }
           
            }
            
			// Se comprueba si hay que poner aviso de socavacion del diente
			if (QueSimula == SIMU_NORMA_ID) {
				if ( (Z < (2 / pow(sin(Angulo * M_PI / 180), 2))) && 
					(-Penetracion < 
					 (1 - Z / 2 * pow(sin(Angulo * M_PI / 180), 2))) ) {
					informa->set_text("    EXISTE SOCAVACION DEL DIENTE");
				} else {
					informa->set_text("");
				}
			}

			break;


        case DATOS_ISO_RADIO_ID:

            // Se ha modificado el radio de cabeza del diente
            ISO_Radio = glui_radio->get_float_val();
		
			// No se admiten valores negativos para el radio
			if (ISO_Radio < 0) {

				glui_radio->set_float_val(0);
				ISO_Radio = 0;

			}

			// La depresion lateral solo existe si existe el radio de cabeza
			if (ISO_Radio) {

				glui_depresion->enable();

			} else {

				glui_depresion->disable();
				glui_depresion->set_float_val(0);
				ISO_Depresion = 0;

			}
            
			// no es posible usar simultaneamente con la deformacion circular
            if ((ISO_Radio == 0) && (ISO_Depresion == 0)) {

                glui_circular->enable();

            } else {

                glui_circular->disable();

            }
            
			break;


        case DATOS_ISO_DEPRE_ID:
           
			// Se ha modificado la depresion lateral del diente
       
			// Solo es posible la depresion si el angulo de presion
			// es mayor que 0
			if ((Angulo > 0) && (Angulo_2 > 0)) {

            	ISO_Depresion = glui_depresion->get_float_val();

			} else {

				ISO_Depresion = 0;
				glui_depresion->set_float_val(ISO_Depresion);

			}

			// no es posible usar simultaneamente la deformacion circular
            if ((ISO_Radio == 0) && (ISO_Depresion == 0)) {

                glui_circular->enable();

            } else {

                glui_circular->disable();

            }
           
			break;


		case DATOS_ANGULO_2_ID:
		
			// Se ha modificado el angulo de presion de la segunda
			// cara del diente asimetrico
			Angulo_2 = glui_Angulo_2->get_float_val();
		
			// no es posible usar simultaneamente con la deformacion circular
			if (Angulo != Angulo_2) {

				glui_circular->disable();

			} else {

				glui_circular->enable();

			}
           
			break;


        case DATOS_FLECHA_ID:
           
			// Se ha modificado la flecha de deformacion del diente
            Flecha = glui_Flecha->get_float_val();
           
			// Si se usa la deformacion circular, no se puede usar
            // las caracteristicas ISO por lo que se deshabilitan
            if (Flecha == 0) {

                glui_iso->enable();
				glui_asimetrico->enable();

            } else {

                glui_iso->disable();
				glui_asimetrico->disable();

            }
           
			break;


        case DATOS_GIRO_ID:
           
			// Se ha modificado el angulo de giro por tallada
            E_incremento = glui_Giro->get_int_val();
		
			// Su valor tiene que ser mayor o igual a 1
			if (E_incremento < 1) {

				glui_Giro->set_int_val(1);
				E_incremento = 1;

			}
			
			break;

			
        case DATOS_ELEMENTOS_ID:
           
			// Se ha seleccionado un objeto para asignarle color
            QueObjeto = glui_Elementos->get_int_val();
           
			// Se asignan los valores del color correspondiente al objeto
            // seleccionado
            switch (QueObjeto) {

                case FONDO_ID:
                    glui_Rojo->set_float_val(ColorFondo[0]);
                    glui_Verde->set_float_val(ColorFondo[1]);
                    glui_Azul->set_float_val(ColorFondo[2]);
        
					break;


                case ENGRANAJE_ID:
                    glui_Rojo->set_float_val(ColorEngranaje[0]);
                    glui_Verde->set_float_val(ColorEngranaje[1]);
                    glui_Azul->set_float_val(ColorEngranaje[2]);
                    
					break;


                case CREMALLERA_ID:
                    glui_Rojo->set_float_val(ColorCremallera[0]);
                    glui_Verde->set_float_val(ColorCremallera[1]);
                    glui_Azul->set_float_val(ColorCremallera[2]);
                    
					break;


                case CIRCULO_FONDO_ID:
                    glui_Rojo->set_float_val(ColorCirFondo[0]);
                    glui_Verde->set_float_val(ColorCirFondo[1]);
                    glui_Azul->set_float_val(ColorCirFondo[2]);
                    
					break;


                case CIRCULO_PASO_ID:
                    glui_Rojo->set_float_val(ColorCirPaso[0]);
                    glui_Verde->set_float_val(ColorCirPaso[1]);
                    glui_Azul->set_float_val(ColorCirPaso[2]);
                    
					break;

            }

			break;


        case COMPONENTE_ROJO_ID:
        
			switch (QueObjeto) {
           
				case FONDO_ID:
               
					ColorFondo[0] = glui_Rojo->get_float_val();
                   
					// selecciona el color negro para limpiar el buffer
                    glClearColor(ColorFondo[0], ColorFondo[1], ColorFondo[2], 
							1);
                   
					// Limpiar la ventana
                    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

                    break;


                case ENGRANAJE_ID:
                   
					ColorEngranaje[0] = glui_Rojo->get_float_val();
                   
					break;


                case CREMALLERA_ID:
                   
					ColorCremallera[0] = glui_Rojo->get_float_val();
                   
					break;


                case CIRCULO_FONDO_ID:
                   
					ColorCirFondo[0] = glui_Rojo->get_float_val();
                   
					break;


                case CIRCULO_PASO_ID:
                   
					ColorCirPaso[0] = glui_Rojo->get_float_val();
                   
					break;

            }

			break;


        case COMPONENTE_VERDE_ID:
        
			switch (QueObjeto) {
           
				case FONDO_ID:
               
					ColorFondo[1] = glui_Verde->get_float_val();
                   
					// selecciona el color negro para limpiar el buffer
                    glClearColor(ColorFondo[0], ColorFondo[1], ColorFondo[2], 
							1);
                   
					// Limpiar la ventana
                    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
                   
					break;


                case ENGRANAJE_ID:

                    ColorEngranaje[1] = glui_Verde->get_float_val();
                    
					break;


                case CREMALLERA_ID:

                    ColorCremallera[1] = glui_Verde->get_float_val();
                    
					break;


                case CIRCULO_FONDO_ID:

                    ColorCirFondo[1] = glui_Verde->get_float_val();
                    
					break;


                case CIRCULO_PASO_ID:

                    ColorCirPaso[1] = glui_Verde->get_float_val();
                   
					break;

            }

            break;


        case COMPONENTE_AZUL_ID:

            switch (QueObjeto) {

                case FONDO_ID:
                    ColorFondo[2] = glui_Azul->get_float_val();
                
					// selecciona el color negro para limpiar el buffer
                    glClearColor(ColorFondo[0], ColorFondo[1], ColorFondo[2], 
							1);
                   
					// Limpiar la ventana
                    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
                   
					break;


                case ENGRANAJE_ID:
                   
					ColorEngranaje[2] = glui_Azul->get_float_val();
                   
					break;


                case CREMALLERA_ID:
                   
					ColorCremallera[2] = glui_Azul->get_float_val();
                   
					break;


                case CIRCULO_FONDO_ID:
                   
					ColorCirFondo[2] = glui_Azul->get_float_val();
                   
					break;


                case CIRCULO_PASO_ID:
                   
					ColorCirPaso[2] = glui_Azul->get_float_val();
                   
					break;

            }

            break;


        case MOVER_ADELANTE_ID:

            // Se ha pulsado el boton de "Siguiente >>"
            Movimiento = 1;
            
			break;


        case MOVER_ATRAS_ID:

            // Se ha pulsado el boton de "<< Anterior"
            Movimiento = -1;

			break;


        case MODO_SIMULAR_ID:

            // Se ha pulsado el boton de "Automatico" y se procede a cambiar
            // la forma de trabaja entre automatico y manual, cambiando el texto
            // del boton, asi como deshabilitanto o habilitando los botones de
            // control manual.
            if (Stop) {

                glui_AutoOnOff->set_name(const_cast<char*> ("Manual"));
                glui_Sig->disable();
                glui_Ant->disable();

            } else {

                glui_AutoOnOff->set_name(const_cast<char*> ("Automatico"));
                glui_Sig->enable();
                glui_Ant->enable();

            }
            Stop = !Stop;
            
			break;


        case SALIR_ID:

            // Se ha pulsado la opcion de salir, con lo que se oculta la ventana
            // del simulador, se destruye el panel de control correspondiente,
            // se muestra la ventana original y se reinician las variables
            // caracteristicas del simulador

            // Reiniciar las variables
            v_glui_control(CLEAR_ID);
            
			// Ocultar la ventana de simulacion
            glutHideWindow();
            
			// Eliminar la v_glui
            v_glui->close();
            
			// Mostrar la glui
            glui->show();
            
			break;


        default:
           
			break;

    }

	// Los limites maximos del radio y la depresion lateral ISO
	// asi como la deformacion experimental de la flecha cambian
	// con los valores de diseo.
	if (Angulo > Angulo_2) {

    	tpAngulo = Angulo * M_PI / 180;

	} else {

    	tpAngulo = Angulo_2 * M_PI / 180;

	}

	// Radio de cabeza ISO
    tpLimite = (Paso * cos(tpAngulo) - 
			2 * (Rc - Rb) * sin(tpAngulo)) / (4 * (1 - sin(tpAngulo)));

	if (tpLimite < 0) {

		// Se evitan valores negativos del radio de cabeza
		tpLimite = 0;

	}

    if (ISO_Radio > tpLimite) {

        ISO_Radio = tpLimite;
        glui_radio->set_float_val(ISO_Radio);

    }

	// Depresion lateral del diente ISO
	if (tpAngulo <= 0) {

		tpLimite = 0;

	} else {

		tpLimite = (Rc - Rb) * sin(tpAngulo) + ISO_Radio * (1 - sin(tpAngulo) -
				cos(tpAngulo));

	}
    
	if (ISO_Depresion > tpLimite) {

    	ISO_Depresion = tpLimite;
        glui_depresion->set_float_val(ISO_Depresion);

    }
	
            // Actualiza en ventana los valores que han cambiado
            v_glui->sync_live();
            glutPostRedisplay();
            
}


//------------------------------------------------------------------------------
//	Nombre:		glui_control
//	Funcion:	Manejar los eventos generados por la ventana de control
//
//	Entrada:
//		control		valor correspondiente a cada control
//------------------------------------------------------------------------------

void glui_control(int control) {

	// Asignamos a la variable el tipo de simulacion a realizar
    QueSimula = control;

    // Se oculta el panel de control
    glui->hide();

    // Se hace visible la ventana de simulacion
    glutShowWindow();


    // Se crea la subventana con los controladores
    v_glui = GLUI_Master.create_glui_subwindow(winIdMain, GLUI_SUBWINDOW_LEFT);


    // Panel de DATOS
    glui_datos = v_glui->add_panel(const_cast<char*> ("Datos"));

    // Numero de dientes del engranaje
    // Es uno de los datos basicos del engranaje
    glui_Dientes = v_glui->add_edittext_to_panel(glui_datos, 
			const_cast<char*> ("Numero de dientes                          "),
            GLUI_EDITTEXT_INT,
            NULL,
            DATOS_DIENTES_ID,
            v_glui_control);
    glui_Dientes->set_int_val(14);
    Z = 14;

    // El modulo se maneja de forma diferente en la simulacion normalizada
    // donde hay que seleccionar unos valores predeterminados a la simulacion
    // experimental que son introducidos por el usuario.
    switch (control) {

        case SIMU_NORMA_ID:

            // Modulo. Es otro de los datos basicos del dise�o
            // Lo dejamos de forma que se pueda solamente elegir entre
            // los valores indicados en la norma UNE 18005:1984 equivalente
            // a la norma ISO 54:1977
            glui_Modulo = v_glui->add_listbox_to_panel(glui_datos,
                    const_cast<char*> ("Modulo                                 "),
                    NULL,
                    DATOS_MODULO_ID,
                    v_glui_control);
            // Los valores recomendados por la norma para el modulo los leemos
            // de la variable MODULO[]
            for (int i = 0; i < 18; i++) {
                glui_Modulo->add_item(i, const_cast<char*> (MODULO[i]));
            }
            glui_Modulo->set_int_val(0);

            break;


        case SIMU_TRAPECIO_ID:

            // Modulo
            glui_Modulo_T = v_glui->add_edittext_to_panel(glui_datos,
                    const_cast<char*>
                    ("Modulo                                      "),
                    GLUI_EDITTEXT_FLOAT,
                    NULL,
                    DATOS_MODULO_ID,
                    v_glui_control);
            glui_Modulo_T->set_float_val(1.0);

            break;

	}

    // Diametro de la circunferencia primitiva
    glui_RPaso = v_glui->add_edittext_to_panel(glui_datos, 
			const_cast<char*> ("Diametro circunferencia primitiva   "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_DIAMETRO_PRI_ID,
            v_glui_control);
    glui_RPaso->set_float_val(14);
    Rp = 7;

    // Diametro de la circunferencia exterior
    glui_RCabeza = v_glui->add_edittext_to_panel(glui_datos, 
			const_cast<char*> ("Diametro exterior                         "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_DIAMETRO_EXT_ID,
            v_glui_control);
    glui_RCabeza->set_float_val(16);
    Rc = 8;

    // Diametro de la circunferencia interior
    glui_RBase = v_glui->add_edittext_to_panel(glui_datos, 
			const_cast<char*> ("Diametro interior                          "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_DIAMETRO_INT_ID,
            v_glui_control);
    glui_RBase->set_float_val(11.5);
    Rb = 5.75;

    // Angulo de presion
    glui_Angulo = v_glui->add_edittext_to_panel(glui_datos, 
			const_cast<char*> ("Angulo de presion                       "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_ANGULO_ID,
            v_glui_control);
    glui_Angulo->set_float_val(20);
    Angulo = 20;

    // Penetracion
    glui_Penetracion = v_glui->add_edittext_to_panel(glui_datos,
            const_cast<char*> ("Coeficiente de desplazamiento      "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_PENETRACION_ID,
            v_glui_control);
    glui_Penetracion->set_float_val(0);

	// Etiqueta de aviso de penetracion en la base del diente
 	if (control == SIMU_NORMA_ID) { 
		v_glui->add_statictext_to_panel(glui_datos, const_cast<char*> (" "));

	    informa = v_glui->add_statictext_to_panel(glui_datos,
				const_cast<char*> (""));
		
		v_glui->add_statictext_to_panel(glui_datos, const_cast<char*> (" "));

		if ( (Z < (2 / pow(sin(Angulo * M_PI / 180), 2))) && 
			(-Penetracion < (1 - Z / 2 * pow(sin(Angulo * M_PI / 180), 2))) ) {
			informa->set_text("    EXISTE SOCAVACION DEL DIENTE");
		}

	}

    // Los datos correspondientes a la ISO 6336 y deformacion experimental
    // si se trata de una simulacion experimental
    if (control == SIMU_TRAPECIO_ID) {


        // Panel de datos correspondientes a la ISO 6336
        glui_iso = v_glui->add_panel(const_cast<char*> ("ISO 6336"));
    	
		// Hacer visible o no el efecto ISO
		glui_ISO = v_glui->add_checkbox_to_panel(glui_iso,
            const_cast<char*> ("Mostrar / Ocultar"),
            &C_ISO,
            -1);
		glui_ISO->set_int_val(1);

        // Radio de la cabeza del diente
        glui_radio = v_glui->add_edittext_to_panel(glui_iso,
                const_cast<char*> ("     Radio de cabeza                   "),
                GLUI_EDITTEXT_FLOAT,
                NULL,
                DATOS_ISO_RADIO_ID,
                v_glui_control);
        glui_radio->set_float_val(0);

        // Depresion lateral del diente
        glui_depresion = v_glui->add_edittext_to_panel(glui_iso,
                const_cast<char*> ("     Depresion lateral del diente    "),
                GLUI_EDITTEXT_FLOAT,
                NULL,
                DATOS_ISO_DEPRE_ID,
                v_glui_control);
        glui_depresion->set_float_val(0);
		glui_depresion->disable();
    

        // Panel de DATOS EXPERIMENTALES
        glui_experimental = v_glui->add_panel(
				const_cast<char*> ("Experimental"));


		glui_asimetrico = v_glui->add_panel_to_panel(glui_experimental, 
				const_cast<char*> ("Diente asimetrico"));
		
		// Hacer visible o no el efecto de asimetria 
		glui_C_Asimetria = v_glui->add_checkbox_to_panel(glui_asimetrico,
            const_cast<char*> ("Mostrar / Ocultar"),
            &C_Asimetria,
            -1);
		glui_C_Asimetria->set_int_val(0);

    	// Angulo de presion de la segunda cara
	    glui_Angulo_2 = v_glui->add_edittext_to_panel(glui_asimetrico, 
			const_cast<char*> (" 2 Angulo "),
            GLUI_EDITTEXT_FLOAT,
            NULL,
            DATOS_ANGULO_2_ID,
            v_glui_control);
	    glui_Angulo_2->set_float_val(20);
   		Angulo_2 = 20;


    	v_glui->add_column_to_panel(glui_experimental, false);
	

		glui_circular = v_glui->add_panel_to_panel(glui_experimental, 
				const_cast<char*> ("Deformacion circular"));

		// Hacer visible o no el efecto de deformacion circular
		glui_C_Flecha = v_glui->add_checkbox_to_panel(glui_circular,
            const_cast<char*> ("Mostrar / Ocultar"),
            &C_Flecha,
            -1);
		glui_C_Flecha->set_int_val(1);

        // Flecha de deformacion del diente
        glui_Flecha = v_glui->add_edittext_to_panel(glui_circular,
                const_cast<char*> ("Flecha "),
                GLUI_EDITTEXT_FLOAT,
                NULL,
                DATOS_FLECHA_ID,
                v_glui_control);
        glui_Flecha->set_float_val(0);


    }


    // Panel extensible para seleccionar el color
    glui_Color = v_glui->add_rollout(const_cast<char*> ("Seleccion del color"),
			true);

    // Dispositivos a los que se le aplica la seleccion del color
    glui_Elementos = v_glui->add_radiogroup_to_panel(glui_Color,
            NULL,
            DATOS_ELEMENTOS_ID,
            v_glui_control);

    // FONDO_ID
    glui_Fondo = v_glui->add_radiobutton_to_group(glui_Elementos,
            const_cast<char*> ("Fondo de la ventana"));
    
	// ENGRANAJE_ID
    glui_Engranaje = v_glui->add_radiobutton_to_group(glui_Elementos,
            const_cast<char*> ("Engranaje"));
   
	// CREMALLERA_ID
    glui_Cremallera = v_glui->add_radiobutton_to_group(glui_Elementos,
            const_cast<char*> ("Cremallera"));
   
	// CIRCULO_FONDO_ID
    glui_C_Fondo = v_glui->add_radiobutton_to_group(glui_Elementos,
            const_cast<char*> ("Circunferencia de fondo"));
   
	// CIRCULO_PASO_ID
    glui_C_Paso = v_glui->add_radiobutton_to_group(glui_Elementos,
            const_cast<char*> ("Circunferencia de paso"));

   
	v_glui->add_column_to_panel(glui_Color, true);
   

	// Se selecciona el color de fondo
    glui_Elementos->set_int_val(0);

    // Colores
    glui_Rojo = v_glui->add_spinner_to_panel(glui_Color,
            const_cast<char*> ("Rojo"),
            GLUI_SPINNER_FLOAT,
            NULL,
            COMPONENTE_ROJO_ID,
            v_glui_control);
    glui_Rojo->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
    glui_Rojo->set_speed(1);

    glui_Verde = v_glui->add_spinner_to_panel(glui_Color,
            const_cast<char*> ("Verde"),
            GLUI_SPINNER_FLOAT,
            NULL,
            COMPONENTE_VERDE_ID,
            v_glui_control);
    glui_Verde->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
    glui_Verde->set_speed(1);

    glui_Azul = v_glui->add_spinner_to_panel(glui_Color,
            const_cast<char*> ("Azul"),
            GLUI_SPINNER_FLOAT,
            NULL,
            COMPONENTE_AZUL_ID,
            v_glui_control);
    glui_Azul->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
    glui_Azul->set_speed(1);


    // Panel de CONTROL
    glui_opciones = v_glui->add_panel(const_cast<char*> ("Opciones"));

    // Angulo que gira el engranaje por cada tallada de la cremallera
    glui_Giro = v_glui->add_edittext_to_panel(glui_opciones,
            const_cast<char*> ("Angulo girado por tallada         "),
            GLUI_EDITTEXT_INT,
            NULL,
            DATOS_GIRO_ID,
            v_glui_control);
    glui_Giro->set_int_val(1);
    E_incremento = 1;

    glui_boton = v_glui->add_panel_to_panel(glui_opciones,
            const_cast<char*> ("Control de movimiento"));

    // Boton para limpiar la ventana de simulacion y reiniciar
    // todo el proceso de simulacion
    glui_Clear = v_glui->add_button_to_panel(glui_boton,
            const_cast<char*> ("Reiniciar"),
            CLEAR_ID,
            v_glui_control);

    // Boton para dar el paso atras en manual
    glui_Ant = v_glui->add_button_to_panel(glui_boton,
            const_cast<char*> ("<< Anterior"),
            MOVER_ATRAS_ID,
            v_glui_control);
    glui_Ant->disable();

    v_glui->add_column_to_panel(glui_boton, false);

    // Boton para cambiar entre proceso automatico y manual
    glui_AutoOnOff = v_glui->add_button_to_panel(glui_boton,
            const_cast<char*> ("Automatico"),
            MODO_SIMULAR_ID,
            v_glui_control);

    // Boton para dar el paso siguiente en manual
    glui_Sig = v_glui->add_button_to_panel(glui_boton,
            const_cast<char*> ("Siguiente >>"),
            MOVER_ADELANTE_ID,
            v_glui_control);

    // Hacer visible o no la circunferencia primitiva
    glui_CPaso = v_glui->add_checkbox_to_panel(glui_opciones,
            const_cast<char*> ("Circunferencia de paso"),
            &C_Paso_ON,
            -1);
    glui_CPaso->set_int_val(1);

    // Hacer visible o no la circunferencia de fondo
    glui_CFondo = v_glui->add_checkbox_to_panel(glui_opciones,
            const_cast<char*> ("Circunferencia de fondo"),
            &C_Fondo_ON,
            -1);
    glui_CFondo->set_int_val(1);

    if (control == SIMU_NORMA_ID) {
    
		// Controladores que permiten el manejor de la imagen (zoom)
        glui_zoom = v_glui->add_panel_to_panel(glui_opciones,
                const_cast<char*> ("Control de imagen"));
       
		v_glui->add_translation_to_panel(glui_zoom,
                const_cast<char*> ("X"),
                GLUI_TRANSLATION_X,
                &trans[0]);
       
		v_glui->add_column_to_panel(glui_zoom, false);
       
		v_glui->add_translation_to_panel(glui_zoom,
                const_cast<char*> ("Y"),
                GLUI_TRANSLATION_Y,
                &trans[1]);
       
		v_glui->add_column_to_panel(glui_zoom, false);
       
		v_glui->add_translation_to_panel(glui_zoom,
                const_cast<char*> ("Z"),
                GLUI_TRANSLATION_Z,
                &trans[2]);

    }


    // Boton para salir del simulador y volver a la ventana original
    // del programa
    v_glui->add_button(const_cast<char*> ("Salir"), SALIR_ID,
            v_glui_control);


    // Determinacion del origen de referencia para la cremallera
    Origen = -Rc - 2 * (Rc - Rb);

    // Calculo del paso vertical
    if (Penetracion > 0) {

        // La penetracion positiva se realiza desplazando
        // el diente desde el eje de rotacion
        C_Paso_Y = 2 * (Rc - Rb) / 10;

    } else {

        // La penetracion negativa se realiza disminuyendo
        // el desplazamiento vertical del eje de rotacion
        C_Paso_Y = (2 * (Rc - Rb) + Penetracion) / 10;

    }

    // Calculo del paso circular
    Paso = 2 * M_PI * Rp / Z;

    // Se calcula y asigna la memoria necesaria para el vector que
    // guarda las posiciones de la cremallera con respecto al eje X
    C_Historia_X = (float *) calloc((int) ceil(360 / E_incremento),
            sizeof (float));

    // El movimiento a realizar es el de subir la cremallera hasta el
    // engranaje
    C_Mueve = SUBIR_ID;

    v_glui->set_main_gfx_window(winIdMain);

    //Redibuja la ventana con el nuevo tama�o
    GLUI_Master.auto_set_viewport();

}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//	Nombre:		gluiInit
//	Funcion:	Inicializa la ventana de control
//------------------------------------------------------------------------------

void gluiInit(void) {

	// Registrar funciones de callback con glui
    GLUI_Master.set_glutIdleFunc(idle);
    GLUI_Master.set_glutDisplayFunc(mainDisplay);
    GLUI_Master.set_glutReshapeFunc(mainReshape);
    GLUI_Master.set_glutKeyboardFunc(keyboard);
    GLUI_Master.set_glutSpecialFunc(pressKey);
    GLUI_Master.set_glutMouseFunc(mousePress);

    // Crear la subventana donde estan los controles
    glui = GLUI_Master.create_glui(const_cast<char*> ("Simulador didactico"));

    // Panel informativo de autores y version
    glui_firma = glui->add_panel(
            const_cast<char*>
            ("UNED. Ingeniera Industrial. Proyecto fin de carrera"));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("Director del proyecto:"));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("     Eduardo Gomez Garcia"));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("     egomez@ind.uned.es"));

    glui->add_column_to_panel(glui_firma, true);
    
	glui->add_statictext_to_panel(glui_firma, const_cast<char*> ("Alumno:"));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("     Daniel Costas Rodriguez"));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("     dcostas@gmail.com"));
   
	glui->add_column_to_panel(glui_firma, true);
   
	glui->add_statictext_to_panel(glui_firma, const_cast<char*> (""));
    glui->add_statictext_to_panel(glui_firma,
            const_cast<char*> ("Version 2.01"));
    glui->add_statictext(const_cast<char*> (""));
    glui->add_separator();
    glui->add_statictext(const_cast<char*> (""));

    // Se escoge la forma de realizar las simulaciones
    glui_datos = glui->add_panel(
            const_cast<char*> ("Forma de realizar la simulacion"));

    glui->add_button_to_panel(glui_datos,
            const_cast<char*> ("Simulacion estandar"),
            SIMU_NORMA_ID,
            glui_control);
    
	glui->add_statictext_to_panel(glui_datos,
            const_cast<char*> ("    Realiza una simulacion del tallado de un engranaje recto siendo"));
   
	glui->add_statictext_to_panel(glui_datos,
            const_cast<char*> ("            conocido el numero de dientes y el diametro primitivo."));

    glui->add_statictext_to_panel(glui_datos, const_cast<char*> (""));
   
	glui->add_button_to_panel(glui_datos,
            const_cast<char*> ("Simulacion experimental"),
            SIMU_TRAPECIO_ID,
            glui_control);
   
	glui->add_statictext_to_panel(glui_datos,
            const_cast<char*> (
            " Realiza una simulacion del tallado de un engranaje recto definiendo")
            );
   
	glui->add_statictext_to_panel(glui_datos,
            const_cast<char*> ("                    los parametros del perfil de la cremallera."));

    glui->add_statictext(const_cast<char*> (""));

    // Boton de finalizar el programa
    glui->add_button(const_cast<char*> ("Salir"), 0, (GLUI_Update_CB) exit);

    informa = glui->add_statictext(const_cast<char*> (""));

    glui->set_main_gfx_window(winIdMain);

}

//------------------------------------------------------------------------------




int main(int argc, char **argv) {

    // inicializar el estado de GLUT
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowPosition(20, 20);
    glutInitWindowSize(1000, 660);

    // Crear y configurar la ventana
    winIdMain = glutCreateWindow(
            "Simulador didactico del tallado de engranajes rectos");

    //	glutIgnoreKeyRepeat(1);
    //	glutSpecialUpFunc(releaseKey);
    glutMotionFunc(activeMouseMotion);

    // Inicializar la ventana glui
    gluiInit();

    //Inicializa las caracteristicas de la ventana
    initDisplay();

    // Ocultar la ventana de simulacion
    glutHideWindow();
    
	// iniciar el procesado de eventos
    glutMainLoop();

    return 0;

}

